from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractThermostatsSimulate


class ThermostatsSimulate(AbstractThermostatsSimulate):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def hvac_mode_adjusted(
        self,
        *,
        device_id: str,
        hvac_mode: str,
        cooling_set_point_celsius: Optional[float] = None,
        cooling_set_point_fahrenheit: Optional[float] = None,
        heating_set_point_celsius: Optional[float] = None,
        heating_set_point_fahrenheit: Optional[float] = None
    ) -> None:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id
        if hvac_mode is not None:
            json_payload["hvac_mode"] = hvac_mode
        if cooling_set_point_celsius is not None:
            json_payload["cooling_set_point_celsius"] = cooling_set_point_celsius
        if cooling_set_point_fahrenheit is not None:
            json_payload["cooling_set_point_fahrenheit"] = cooling_set_point_fahrenheit
        if heating_set_point_celsius is not None:
            json_payload["heating_set_point_celsius"] = heating_set_point_celsius
        if heating_set_point_fahrenheit is not None:
            json_payload["heating_set_point_fahrenheit"] = heating_set_point_fahrenheit

        self.client.post("/thermostats/simulate/hvac_mode_adjusted", json=json_payload)

        return None

    def temperature_reached(
        self,
        *,
        device_id: str,
        temperature_celsius: Optional[float] = None,
        temperature_fahrenheit: Optional[float] = None
    ) -> None:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id
        if temperature_celsius is not None:
            json_payload["temperature_celsius"] = temperature_celsius
        if temperature_fahrenheit is not None:
            json_payload["temperature_fahrenheit"] = temperature_fahrenheit

        self.client.post("/thermostats/simulate/temperature_reached", json=json_payload)

        return None



from typing import Any, Dict, Optional, Union
import niquests as requests
from typing_extensions import Self
from urllib3.util import Retry

from .auth import get_auth_headers_for_multi_workspace_personal_access_token
from .constants import LTS_VERSION
from .options import get_endpoint
from .client import SeamHttpClient
from .models import AbstractSeamMultiWorkspace
from .routes.workspaces import Workspaces


class WorkspacesProxy:
    """Proxy to expose only the 'create' and 'list' methods of Workspaces."""

    def __init__(self, workspaces):
        self._workspaces = workspaces

    def list(self, **kwargs):
        return self._workspaces.list(**kwargs)

    def create(self, **kwargs):
        return self._workspaces.create(**kwargs)


class SeamMultiWorkspace(AbstractSeamMultiWorkspace):
    """
    Seam class used to interact with Seam API without being scoped to a specific workspace.

    This class provides methods to authenticate and interact with Seam API endpoints
    that can operate without being tied to a specific workspace. It supports operations such as creating and listing workspaces.

    :cvar lts_version: The long-term support (LTS) version of the Seam
        Python SDK
    :vartype lts_version: str
    :ivar wait_for_action_attempt: Controls whether to wait for an action
        attempt to complete
    :vartype wait_for_action_attempt: Union[bool, Dict[str, float]]
    :ivar client: The HTTP client used for making API requests
    :vartype client: SeamHttpClient
    :ivar workspaces: Proxy to access workspace-related operations
    :vartype workspaces: WorkspacesProxy
    """

    lts_version: str = LTS_VERSION

    def __init__(
        self,
        personal_access_token: str,
        *,
        endpoint: Optional[str] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = True,
        retries: Optional[Retry] = None,
    ):
        """
        Initialize a SeamMultiWorkspace client instance.

        This method sets up the SeamMultiWorkspace client with the provided personal access token
        and configuration options.

        :param personal_access_token: A personal access token for
            authenticating with Seam
        :type personal_access_token: str
        :param endpoint: The custom API endpoint URL. If not provided,
            the default Seam API endpoint will be used
        :type endpoint: Optional[str]
        :param wait_for_action_attempt: Controls whether to wait for an
            action attempt to complete. Can be a boolean or a dictionary with
            'timeout' and 'poll_interval' keys
        :type wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]]
        :param retries: Configuration for retry behavior on failed requests
        :type retries: Optional[urllib3.util.Retry]

        :raises SeamInvalidTokenError: If the provided personal access token format is invalid
        """

        self.lts_version = SeamMultiWorkspace.lts_version
        self.wait_for_action_attempt = wait_for_action_attempt
        auth_headers = get_auth_headers_for_multi_workspace_personal_access_token(
            personal_access_token
        )
        endpoint = get_endpoint(endpoint)

        self.client = SeamHttpClient(
            base_url=endpoint,
            auth_headers=auth_headers,
            retries=retries,
        )

        defaults = {"wait_for_action_attempt": wait_for_action_attempt}

        self._workspaces = Workspaces(client=self.client, defaults=defaults)
        self.workspaces = WorkspacesProxy(self._workspaces)

    @classmethod
    def from_personal_access_token(
        cls,
        personal_access_token: str,
        *,
        endpoint: Optional[str] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = True,
        retries: Optional[Retry] = None,
    ) -> Self:
        """
        Create a SeamMultiWorkspace instance using a personal access token.

        This class method is a convenience constructor for creating a SeamMultiWorkspace instance
        authenticated with a personal access token.

        :param personal_access_token: The personal access token for authenticating with Seam
        :type personal_access_token: str
        :param endpoint: The custom API endpoint URL. If not provided, the default Seam API endpoint will be used
        :type endpoint: Optional[str]
        :param wait_for_action_attempt: Controls whether to wait for an
            action attempt to complete. Can be a boolean or a dictionary with
            'timeout' and 'poll_interval' keys
        :type wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]]
        :param retries: Configuration for retry behavior on failed requests
        :type retries: Optional[urllib3.util.Retry]
        :return: A new instance of the SeamMultiWorkspace class
            authenticated with the provided personal access token
        :rtype: Self

        :Example:

        >>> seam = SeamMultiWorkspace.from_personal_access_token("your-personal-access-token-here")
        """

        return cls(
            personal_access_token=personal_access_token,
            endpoint=endpoint,
            wait_for_action_attempt=wait_for_action_attempt,
            retries=retries,
        )



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractConnectWebviews, ConnectWebview


class ConnectWebviews(AbstractConnectWebviews):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def create(
        self,
        *,
        accepted_capabilities: Optional[List[str]] = None,
        accepted_providers: Optional[List[str]] = None,
        automatically_manage_new_devices: Optional[bool] = None,
        custom_metadata: Optional[Dict[str, Any]] = None,
        custom_redirect_failure_url: Optional[str] = None,
        custom_redirect_url: Optional[str] = None,
        customer_id: Optional[str] = None,
        device_selection_mode: Optional[str] = None,
        provider_category: Optional[str] = None,
        wait_for_device_creation: Optional[bool] = None
    ) -> ConnectWebview:
        json_payload = {}

        if accepted_capabilities is not None:
            json_payload["accepted_capabilities"] = accepted_capabilities
        if accepted_providers is not None:
            json_payload["accepted_providers"] = accepted_providers
        if automatically_manage_new_devices is not None:
            json_payload["automatically_manage_new_devices"] = (
                automatically_manage_new_devices
            )
        if custom_metadata is not None:
            json_payload["custom_metadata"] = custom_metadata
        if custom_redirect_failure_url is not None:
            json_payload["custom_redirect_failure_url"] = custom_redirect_failure_url
        if custom_redirect_url is not None:
            json_payload["custom_redirect_url"] = custom_redirect_url
        if customer_id is not None:
            json_payload["customer_id"] = customer_id
        if device_selection_mode is not None:
            json_payload["device_selection_mode"] = device_selection_mode
        if provider_category is not None:
            json_payload["provider_category"] = provider_category
        if wait_for_device_creation is not None:
            json_payload["wait_for_device_creation"] = wait_for_device_creation

        res = self.client.post("/connect_webviews/create", json=json_payload)

        return ConnectWebview.from_dict(res["connect_webview"])

    def delete(self, *, connect_webview_id: str) -> None:
        json_payload = {}

        if connect_webview_id is not None:
            json_payload["connect_webview_id"] = connect_webview_id

        self.client.post("/connect_webviews/delete", json=json_payload)

        return None

    def get(self, *, connect_webview_id: str) -> ConnectWebview:
        json_payload = {}

        if connect_webview_id is not None:
            json_payload["connect_webview_id"] = connect_webview_id

        res = self.client.post("/connect_webviews/get", json=json_payload)

        return ConnectWebview.from_dict(res["connect_webview"])

    def list(
        self,
        *,
        custom_metadata_has: Optional[Dict[str, Any]] = None,
        customer_ids: Optional[List[str]] = None,
        limit: Optional[float] = None,
        page_cursor: Optional[str] = None,
        user_identifier_key: Optional[str] = None
    ) -> List[ConnectWebview]:
        json_payload = {}

        if custom_metadata_has is not None:
            json_payload["custom_metadata_has"] = custom_metadata_has
        if customer_ids is not None:
            json_payload["customer_ids"] = customer_ids
        if limit is not None:
            json_payload["limit"] = limit
        if page_cursor is not None:
            json_payload["page_cursor"] = page_cursor
        if user_identifier_key is not None:
            json_payload["user_identifier_key"] = user_identifier_key

        res = self.client.post("/connect_webviews/list", json=json_payload)

        return [ConnectWebview.from_dict(item) for item in res["connect_webviews"]]



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractSpaces, Space


class Spaces(AbstractSpaces):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def add_acs_entrances(self, *, acs_entrance_ids: List[str], space_id: str) -> None:
        json_payload = {}

        if acs_entrance_ids is not None:
            json_payload["acs_entrance_ids"] = acs_entrance_ids
        if space_id is not None:
            json_payload["space_id"] = space_id

        self.client.post("/spaces/add_acs_entrances", json=json_payload)

        return None

    def add_devices(self, *, device_ids: List[str], space_id: str) -> None:
        json_payload = {}

        if device_ids is not None:
            json_payload["device_ids"] = device_ids
        if space_id is not None:
            json_payload["space_id"] = space_id

        self.client.post("/spaces/add_devices", json=json_payload)

        return None

    def create(
        self,
        *,
        name: str,
        acs_entrance_ids: Optional[List[str]] = None,
        device_ids: Optional[List[str]] = None
    ) -> Space:
        json_payload = {}

        if name is not None:
            json_payload["name"] = name
        if acs_entrance_ids is not None:
            json_payload["acs_entrance_ids"] = acs_entrance_ids
        if device_ids is not None:
            json_payload["device_ids"] = device_ids

        res = self.client.post("/spaces/create", json=json_payload)

        return Space.from_dict(res["space"])

    def delete(self, *, space_id: str) -> None:
        json_payload = {}

        if space_id is not None:
            json_payload["space_id"] = space_id

        self.client.post("/spaces/delete", json=json_payload)

        return None

    def get(self, *, space_id: str) -> Space:
        json_payload = {}

        if space_id is not None:
            json_payload["space_id"] = space_id

        res = self.client.post("/spaces/get", json=json_payload)

        return Space.from_dict(res["space"])

    def list(
        self,
    ) -> List[Space]:
        json_payload = {}

        res = self.client.post("/spaces/list", json=json_payload)

        return [Space.from_dict(item) for item in res["spaces"]]

    def remove_acs_entrances(
        self, *, acs_entrance_ids: List[str], space_id: str
    ) -> None:
        json_payload = {}

        if acs_entrance_ids is not None:
            json_payload["acs_entrance_ids"] = acs_entrance_ids
        if space_id is not None:
            json_payload["space_id"] = space_id

        self.client.post("/spaces/remove_acs_entrances", json=json_payload)

        return None

    def remove_devices(self, *, device_ids: List[str], space_id: str) -> None:
        json_payload = {}

        if device_ids is not None:
            json_payload["device_ids"] = device_ids
        if space_id is not None:
            json_payload["space_id"] = space_id

        self.client.post("/spaces/remove_devices", json=json_payload)

        return None

    def update(self, *, space_id: str, name: Optional[str] = None) -> Space:
        json_payload = {}

        if space_id is not None:
            json_payload["space_id"] = space_id
        if name is not None:
            json_payload["name"] = name

        res = self.client.post("/spaces/update", json=json_payload)

        return Space.from_dict(res["space"])



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractCustomers, MagicLink


class Customers(AbstractCustomers):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def create_portal(
        self,
        *,
        features: Optional[Dict[str, Any]] = None,
        is_embedded: Optional[bool] = None,
        customer_data: Optional[Dict[str, Any]] = None
    ) -> MagicLink:
        json_payload = {}

        if features is not None:
            json_payload["features"] = features
        if is_embedded is not None:
            json_payload["is_embedded"] = is_embedded
        if customer_data is not None:
            json_payload["customer_data"] = customer_data

        res = self.client.post("/customers/create_portal", json=json_payload)

        return MagicLink.from_dict(res["magic_link"])

    def push_data(
        self,
        *,
        customer_key: str,
        access_grants: Optional[List[Dict[str, Any]]] = None,
        bookings: Optional[List[Dict[str, Any]]] = None,
        buildings: Optional[List[Dict[str, Any]]] = None,
        common_areas: Optional[List[Dict[str, Any]]] = None,
        facilities: Optional[List[Dict[str, Any]]] = None,
        guests: Optional[List[Dict[str, Any]]] = None,
        listings: Optional[List[Dict[str, Any]]] = None,
        properties: Optional[List[Dict[str, Any]]] = None,
        property_listings: Optional[List[Dict[str, Any]]] = None,
        reservations: Optional[List[Dict[str, Any]]] = None,
        residents: Optional[List[Dict[str, Any]]] = None,
        rooms: Optional[List[Dict[str, Any]]] = None,
        spaces: Optional[List[Dict[str, Any]]] = None,
        tenants: Optional[List[Dict[str, Any]]] = None,
        units: Optional[List[Dict[str, Any]]] = None,
        user_identities: Optional[List[Dict[str, Any]]] = None,
        users: Optional[List[Dict[str, Any]]] = None
    ) -> None:
        json_payload = {}

        if customer_key is not None:
            json_payload["customer_key"] = customer_key
        if access_grants is not None:
            json_payload["access_grants"] = access_grants
        if bookings is not None:
            json_payload["bookings"] = bookings
        if buildings is not None:
            json_payload["buildings"] = buildings
        if common_areas is not None:
            json_payload["common_areas"] = common_areas
        if facilities is not None:
            json_payload["facilities"] = facilities
        if guests is not None:
            json_payload["guests"] = guests
        if listings is not None:
            json_payload["listings"] = listings
        if properties is not None:
            json_payload["properties"] = properties
        if property_listings is not None:
            json_payload["property_listings"] = property_listings
        if reservations is not None:
            json_payload["reservations"] = reservations
        if residents is not None:
            json_payload["residents"] = residents
        if rooms is not None:
            json_payload["rooms"] = rooms
        if spaces is not None:
            json_payload["spaces"] = spaces
        if tenants is not None:
            json_payload["tenants"] = tenants
        if units is not None:
            json_payload["units"] = units
        if user_identities is not None:
            json_payload["user_identities"] = user_identities
        if users is not None:
            json_payload["users"] = users

        self.client.post("/customers/push_data", json=json_payload)

        return None



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractAcsEntrances, AcsEntrance, AcsCredential


class AcsEntrances(AbstractAcsEntrances):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def get(self, *, acs_entrance_id: str) -> AcsEntrance:
        json_payload = {}

        if acs_entrance_id is not None:
            json_payload["acs_entrance_id"] = acs_entrance_id

        res = self.client.post("/acs/entrances/get", json=json_payload)

        return AcsEntrance.from_dict(res["acs_entrance"])

    def grant_access(
        self,
        *,
        acs_entrance_id: str,
        acs_user_id: Optional[str] = None,
        user_identity_id: Optional[str] = None
    ) -> None:
        json_payload = {}

        if acs_entrance_id is not None:
            json_payload["acs_entrance_id"] = acs_entrance_id
        if acs_user_id is not None:
            json_payload["acs_user_id"] = acs_user_id
        if user_identity_id is not None:
            json_payload["user_identity_id"] = user_identity_id

        self.client.post("/acs/entrances/grant_access", json=json_payload)

        return None

    def list(
        self,
        *,
        access_grant_id: Optional[str] = None,
        access_method_id: Optional[str] = None,
        acs_credential_id: Optional[str] = None,
        acs_system_id: Optional[str] = None,
        location_id: Optional[str] = None,
        space_id: Optional[str] = None
    ) -> List[AcsEntrance]:
        json_payload = {}

        if access_grant_id is not None:
            json_payload["access_grant_id"] = access_grant_id
        if access_method_id is not None:
            json_payload["access_method_id"] = access_method_id
        if acs_credential_id is not None:
            json_payload["acs_credential_id"] = acs_credential_id
        if acs_system_id is not None:
            json_payload["acs_system_id"] = acs_system_id
        if location_id is not None:
            json_payload["location_id"] = location_id
        if space_id is not None:
            json_payload["space_id"] = space_id

        res = self.client.post("/acs/entrances/list", json=json_payload)

        return [AcsEntrance.from_dict(item) for item in res["acs_entrances"]]

    def list_credentials_with_access(
        self, *, acs_entrance_id: str, include_if: Optional[List[str]] = None
    ) -> List[AcsCredential]:
        json_payload = {}

        if acs_entrance_id is not None:
            json_payload["acs_entrance_id"] = acs_entrance_id
        if include_if is not None:
            json_payload["include_if"] = include_if

        res = self.client.post(
            "/acs/entrances/list_credentials_with_access", json=json_payload
        )

        return [AcsCredential.from_dict(item) for item in res["acs_credentials"]]



from typing import Callable, Dict, Any, Tuple, Generator, List
from .client import SeamHttpClient
from niquests import Response, JSONDecodeError
from .pagination import Pagination


class SeamPaginator:
    """
    Handles pagination for API list endpoints.

    Iterates through pages of results returned by a callable function.
    """

    _FIRST_PAGE = "FIRST_PAGE"

    def __init__(
        self,
        client: SeamHttpClient,
        request: Callable,
        params: Dict[str, Any] = None,
    ):
        """
        Initializes the Paginator.

        Args:
            request: The function to call to fetch a page of data.
            http_client: The Seam HTTP client used in the request.
            params: Initial parameters to pass to the callable function.
        """
        self._request = request
        self.client = client
        self._params = params or {}
        self._pagination_cache: Dict[str, Pagination] = {}

    def first_page(self) -> Tuple[List[Any], Pagination | None]:
        """Fetches the first page of results."""
        self.client.hooks["response"].append(
            lambda response: self._cache_pagination(response, self._FIRST_PAGE)
        )
        data = self._request(**self._params)
        self.client.hooks["response"].pop()

        pagination = self._pagination_cache.get(self._FIRST_PAGE)

        return data, pagination

    def next_page(
        self, next_page_cursor: str, /
    ) -> Tuple[List[Any], Pagination | None]:
        """Fetches the next page of results using a cursor."""
        if not next_page_cursor:
            raise ValueError("Cannot get the next page with a null next_page_cursor.")

        params = {
            **self._params,
            "page_cursor": next_page_cursor,
        }

        self.client.hooks["response"].append(
            lambda response: self._cache_pagination(response, next_page_cursor)
        )
        data = self._request(**params)
        self.client.hooks["response"].pop()

        pagination = self._pagination_cache.get(next_page_cursor)

        return data, pagination

    def flatten_to_list(self) -> List[Any]:
        """Fetches all pages and returns all items as a single list."""
        all_items = []
        current_items, pagination = self.first_page()

        if current_items:
            all_items.extend(current_items)

        while pagination.has_next_page:
            current_items, pagination = self.next_page(pagination.next_page_cursor)
            if current_items:
                all_items.extend(current_items)

        return all_items

    def flatten(self) -> Generator[Any, None, None]:
        """Fetches all pages and yields items one by one using a generator."""
        current_items, pagination = self.first_page()
        if current_items:
            yield from current_items

        while pagination and pagination.has_next_page and pagination.next_page_cursor:
            current_items, pagination = self.next_page(pagination.next_page_cursor)
            if current_items:
                yield from current_items

    def _cache_pagination(self, response: Response, page_key: str) -> None:
        """Extracts pagination dict from response, creates Pagination object, and caches it."""
        try:
            response_json = response.json()
            pagination = response_json.get("pagination", {})
        except JSONDecodeError:
            pagination = {}

        if isinstance(pagination, dict):
            self._pagination_cache[page_key] = Pagination(
                has_next_page=pagination.get("has_next_page", False),
                next_page_cursor=pagination.get("next_page_cursor"),
                next_page_url=pagination.get("next_page_url"),
            )



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractAccessCodesSimulate, UnmanagedAccessCode


class AccessCodesSimulate(AbstractAccessCodesSimulate):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def create_unmanaged_access_code(
        self, *, code: str, device_id: str, name: str
    ) -> UnmanagedAccessCode:
        json_payload = {}

        if code is not None:
            json_payload["code"] = code
        if device_id is not None:
            json_payload["device_id"] = device_id
        if name is not None:
            json_payload["name"] = name

        res = self.client.post(
            "/access_codes/simulate/create_unmanaged_access_code", json=json_payload
        )

        return UnmanagedAccessCode.from_dict(res["access_code"])



from typing import Any, Optional, Union, Dict, Callable
from typing_extensions import Self
from urllib3.util.retry import Retry

from .constants import LTS_VERSION
from .parse_options import parse_options
from .routes import Routes
from .models import AbstractSeam
from .client import SeamHttpClient
from .paginator import SeamPaginator


class Seam(AbstractSeam):
    """Main class for interacting with the Seam API.

    This class provides methods to authenticate and interact with various
    Seam API endpoints,
    including devices, access codes, action_attempts, and more. It supports authentication via API key or personal access token.

    :cvar lts_version: The long-term support (LTS) version of the Seam
        Python SDK
    :vartype lts_version: str
    :ivar defaults: Default settings for API requests
    :vartype defaults: Dict[str, Any]
    :ivar client: The HTTP client used for making API requests
    :vartype client: SeamHttpClient
    :ivar wait_for_action_attempt: Controls whether to wait for an action
        attempt to complete
    :vartype wait_for_action_attempt: Union[bool, Dict[str, float]]

    For more information about the Seam API, visit https://docs.seam.co/
    """

    lts_version: str = LTS_VERSION

    def __init__(
        self,
        api_key: Optional[str] = None,
        *,
        personal_access_token: Optional[str] = None,
        workspace_id: Optional[str] = None,
        endpoint: Optional[str] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = True,
        retries: Optional[Retry] = None,
    ):
        """Initialize a Seam client instance.

        This method sets up the Seam client with the provided authentication credentials and configuration options.
        It supports two authentication methods: API key or personal access token.

        :param api_key: The API key for authenticating with Seam. Mutually
            exclusive with personal_access_token
        :type api_key: Optional[str]
        :param personal_access_token: A personal access token for
            authenticating with Seam. Mutually exclusive with api_key
        :type personal_access_token: Optional[str]
        :param workspace_id: The ID of the workspace to interact with.
            Required when using a personal access token
        :type workspace_id: Optional[str]
        :param endpoint: The custom API endpoint URL. If not provided, the
            default Seam API endpoint will be used
        :type endpoint: Optional[str]
        :param wait_for_action_attempt: Controls whether to wait for an
            action attempt to complete. Can be a boolean or a dictionary with
            'timeout' and 'poll_interval' keys
        :type wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]]
        :param retries: Configuration for retry behavior on failed requests
        :type retries: Optional[urllib3.util.Retry]

        :raises SeamInvalidOptionsError: If neither api_key nor
            personal_access_token is provided, or if workspace_id is missing
            when using a personal access token
        :raises SeamInvalidTokenError: If the provided API key or personal
            access token format is invalid
        """

        self.lts_version = Seam.lts_version
        self.wait_for_action_attempt = wait_for_action_attempt
        auth_headers, endpoint = parse_options(
            api_key=api_key,
            personal_access_token=personal_access_token,
            workspace_id=workspace_id,
            endpoint=endpoint,
        )
        self.defaults = {"wait_for_action_attempt": wait_for_action_attempt}

        self.client = SeamHttpClient(
            base_url=endpoint, auth_headers=auth_headers, retries=retries
        )

        Routes.__init__(self, client=self.client, defaults=self.defaults)

    def create_paginator(
        self, request: Callable, params: Optional[Dict[str, Any]] = None, /
    ) -> SeamPaginator:
        """
        Creates a Paginator instance for iterating through list endpoints.

        This is a helper method to simplify the process of paginating through
        API results.

        Args:
            request: The API route method function to call for fetching pages
                (e.g., connected_accounts.list).
            params: Optional dictionary of initial parameters to pass to the request
                function.

        Returns:
            An initialized Paginator object ready to fetch pages.

        Example:
            >>> connected_accounts_paginator = seam.create_paginator(seam.connected_accounts.list)
            >>> for connected_account in connected_accounts_paginator.flatten():
            >>>     print(connected_account.account_type_display_name)
        """
        return SeamPaginator(self.client, request, params)

    @classmethod
    def from_api_key(
        cls,
        api_key: str,
        *,
        endpoint: Optional[str] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = True,
        retries: Optional[Retry] = None,
    ) -> Self:
        """Create a Seam instance using an API key.

        This class method is a convenience constructor for creating a Seam instance authenticated with an API key.

        :param api_key: The API key for authenticating with Seam. Mutually
            exclusive with personal_access_token
        :type api_key: str
        :param endpoint: The custom API endpoint URL. If not provided, the
            default Seam API endpoint will be used
        :type endpoint: Optional[str]
        :param wait_for_action_attempt: Controls whether to wait for an
            action attempt to complete. Can be a boolean or a dictionary with
            'timeout' and 'poll_interval' keys
        :type wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]]
        :return: A new instance of the Seam class authenticated with the
            provided API key
        :rtype: Self

        :Example:

        >>> seam = Seam.from_api_key("your-api-key-here")
        """
        return cls(
            api_key,
            endpoint=endpoint,
            wait_for_action_attempt=wait_for_action_attempt,
            retries=retries,
        )

    @classmethod
    def from_personal_access_token(
        cls,
        personal_access_token: str,
        workspace_id: str,
        *,
        endpoint: Optional[str] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = True,
        retries: Optional[Retry] = None,
    ) -> Self:
        """Create a Seam instance using a personal access token.

        This class method is a convenience constructor for creating a Seam
        instance authenticated with a personal access token.

        :param personal_access_token: The personal access token for
            authenticating with Seam
        :type personal_access_token: str
        :param workspace_id: The ID of the workspace to interact with
        :type workspace_id: str
        :param endpoint: The custom API endpoint URL. If not provided, the
            default Seam API endpoint will be used
        :type endpoint: Optional[str]
        :param wait_for_action_attempt: Controls whether to wait for an
            action attempt to complete. Can be a boolean or a dictionary with
            'timeout' and 'poll_interval' keys
        :type wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]]
        :return: A new instance of the Seam class authenticated with the
            provided personal access token
        :rtype: Self

        :Example:

        >>> seam = Seam.from_personal_access_token("your-token-here", "workspace-id")
        """
        return cls(
            personal_access_token=personal_access_token,
            workspace_id=workspace_id,
            endpoint=endpoint,
            wait_for_action_attempt=wait_for_action_attempt,
            retries=retries,
        )



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractUserIdentitiesEnrollmentAutomations, EnrollmentAutomation


class UserIdentitiesEnrollmentAutomations(AbstractUserIdentitiesEnrollmentAutomations):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def delete(self, *, enrollment_automation_id: str) -> None:
        json_payload = {}

        if enrollment_automation_id is not None:
            json_payload["enrollment_automation_id"] = enrollment_automation_id

        self.client.post(
            "/user_identities/enrollment_automations/delete", json=json_payload
        )

        return None

    def get(self, *, enrollment_automation_id: str) -> EnrollmentAutomation:
        json_payload = {}

        if enrollment_automation_id is not None:
            json_payload["enrollment_automation_id"] = enrollment_automation_id

        res = self.client.post(
            "/user_identities/enrollment_automations/get", json=json_payload
        )

        return EnrollmentAutomation.from_dict(res["enrollment_automation"])

    def launch(
        self,
        *,
        credential_manager_acs_system_id: str,
        user_identity_id: str,
        acs_credential_pool_id: Optional[str] = None,
        create_credential_manager_user: Optional[bool] = None,
        credential_manager_acs_user_id: Optional[str] = None
    ) -> EnrollmentAutomation:
        json_payload = {}

        if credential_manager_acs_system_id is not None:
            json_payload["credential_manager_acs_system_id"] = (
                credential_manager_acs_system_id
            )
        if user_identity_id is not None:
            json_payload["user_identity_id"] = user_identity_id
        if acs_credential_pool_id is not None:
            json_payload["acs_credential_pool_id"] = acs_credential_pool_id
        if create_credential_manager_user is not None:
            json_payload["create_credential_manager_user"] = (
                create_credential_manager_user
            )
        if credential_manager_acs_user_id is not None:
            json_payload["credential_manager_acs_user_id"] = (
                credential_manager_acs_user_id
            )

        res = self.client.post(
            "/user_identities/enrollment_automations/launch", json=json_payload
        )

        return EnrollmentAutomation.from_dict(res["enrollment_automation"])

    def list(self, *, user_identity_id: str) -> List[EnrollmentAutomation]:
        json_payload = {}

        if user_identity_id is not None:
            json_payload["user_identity_id"] = user_identity_id

        res = self.client.post(
            "/user_identities/enrollment_automations/list", json=json_payload
        )

        return [
            EnrollmentAutomation.from_dict(item)
            for item in res["enrollment_automations"]
        ]
